#!/usr/bin/python
# -*- encoding: utf-8; py-indent-offset: 4 -*-
#
###############################################################################
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
###############################################################################

###############################################################################
# Author: Viktor Kertesz (vkertesz@albacomp.hu)
# Changed by: Bernd Mueller (bernd.mueller@de.lisega.com)
###############################################################################

from .agent_based_api.v1 import register, SNMPTree, contains, Service, Metric, Result, State

def parse_sonicwall_connections(string_table):
    return {
            'max': int(string_table[0][0]),
            'current': int(string_table[0][1]),
            }

def discovery_sonicwall_connections(section):
    yield Service()

def check_sonicwall_connections(params, section):
    warn, crit = params.get("warn"), params.get("crit")
    conns, max_conns = int(section['current']), int(section['max'])
    warn_conns, crit_conns = (max_conns/100*warn, max_conns/100*crit)
    percent = int(100 * conns/max_conns)
    yield Metric(
        "connection_percent",
        percent,
        levels=(warn, crit),
        boundaries=(0, 100),
        )

    yield Metric(
        "connection_count",
        conns,
        levels=(warn_conns, crit_conns),
        boundaries=(0, max_conns),
        )

    yield Result(state=(State.CRIT if conns >= crit_conns else
        State.WARN if conns >= warn_conns else State.OK),
        summary = f"Connections: {conns} out of {max_conns} ({percent} percent)")

register.snmp_section(
    name = "sonicwall_connections",
    detect = contains(".1.3.6.1.2.1.1.1.0", "sonicwall"),
    fetch=SNMPTree(
        base = '.1.3.6.1.4.1.8741.1.3.1',
        oids=[
            '1.0', # SONICWALL-FIREWALL-IP-STATISTICS-MIB::sonicMaxConnCacheEntries
            '2.0', # SONICWALL-FIREWALL-IP-STATISTICS-MIB::sonicCurrentConnCacheEntries
            ]),
        parse_function=parse_sonicwall_connections,
)

register.check_plugin(
        name="sonicwall_connections",
        service_name="Sonnicwall Connections",
        discovery_function=discovery_sonicwall_connections,
        check_function=check_sonicwall_connections,
        check_default_parameters={"warn": 80.0, "crit": 95.0},
        check_ruleset_name='sonicwall_connections',
)
