#!/usr/bin/python
# -*- encoding: utf-8; py-indent-offset: 4 -*-
#
###############################################################################
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
###############################################################################

###############################################################################
# Author: Viktor Kertesz (vkertesz@albacomp.hu)
# Changed by: Bernd Mueller (bernd.mueller@de.lisega.com)
###############################################################################

from .agent_based_api.v1 import register, SNMPTree, contains, Service, Metric, Result, State

def parse_sonicwall_mem(string_table):
    return {
            'mem': int(string_table[0][0]),
            }

def discovery_sonicwall_mem(section):
    yield Service()

def check_sonicwall_mem(params, section):
    warn, crit = params.get("warn"), params.get("crit")
    mem = int(section['mem'])
    yield Metric(
        "mem_usage",
        mem,
        levels=(warn, crit),
        boundaries=(0, 100),
        )

    yield Result(state=(State.CRIT if mem >= crit else
        State.WARN if mem >= warn else State.OK),
        summary = f"MEM is {mem} percent")

register.snmp_section(
    name = "sonicwall_mem",
    detect = contains(".1.3.6.1.2.1.1.1.0", "sonicwall"),
    fetch=SNMPTree(
        base = '.1.3.6.1.4.1.8741.1.3.1.4',
        oids=[
            '0', # SONICWALL-FIREWALL-IP-STATISTICS-MIB::sonicCurrentRAMUtil
            ]),
        parse_function=parse_sonicwall_mem,
)

register.check_plugin(
        name="sonicwall_mem",
        service_name="Sonnicwall MEM Usage",
        discovery_function=discovery_sonicwall_mem,
        check_function=check_sonicwall_mem,
        check_default_parameters={"warn": 80.0, "crit": 95.0},
        check_ruleset_name='sonicwall_mem',
)
