#!/usr/bin/python
# -*- encoding: utf-8; py-indent-offset: 4 -*-
#
###############################################################################
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
###############################################################################

###############################################################################
# Author: Viktor Kertesz (vkertesz@albacomp.hu)
# Changed by: Bernd Mueller (bernd.mueller@de.lisega.com)
###############################################################################

from cmk.base.check_api import get_rate
from .agent_based_api.v1 import register, SNMPTree, contains, Service, Metric, Result, State, render

# Soll man nicht, aber wenn CMK sich nicht daran hälrt, warum ich????
import time

# Muss ich definieren, da get_rate ohne nicht funktioniert.
SKIP = None

#[['AA', '0', '0', '169.254.123.11'],
# ['AAB', '0', '0', '169.254.123.1'],
# ['AAB', '151763', '24859', '169.254.123.1']]

# Need to remove double entries, but keep the enc and dec bytes

def parse_sonicwall_vpn(string_table):
    string_table.sort()
    parsed = {}
    for v in string_table:
        if v[0] not in parsed:
            parsed[v[0]] = v
            continue
        for i, n in enumerate(v[1:2], 1):
            parsed[v[0]][i] = str(int(parsed[v[0]][i]) + int(n))
    return parsed

def discovery_sonicwall_vpn(section):
    for vpn in section:
        yield Service(item=vpn)

def check_sonicwall_vpn(item, section):
    temp = []
    for i in section[item]:
        temp.append(i)
    this_time = int(time.time())
    encryptedbytes=int(temp[1])
    decryptedbytes=int(temp[2])
    encrate = int(get_rate("vpn.enc.%s" % item, this_time, int(encryptedbytes), onwrap=SKIP))
    decrate = int(get_rate("vpn.dec.%s" % item, this_time, int(decryptedbytes), onwrap=SKIP))
    yield Metric("if_in_bps", encrate)
    yield Metric("if_out_bps", decrate)
    yield Result(state=State.OK, summary=f"(%s/%s) active" % (temp[0], temp[3]))

register.snmp_section(
    name = "sonicwall_vpn",
    detect = contains(".1.3.6.1.2.1.1.1.0", "sonicwall"),
    fetch=SNMPTree(
        base = '.1.3.6.1.4.1.8741.1.3.2.1.1.1',
        oids=[
            '14',# SONICWALL-FIREWALL-IP-STATISTICS-MIB::sonicSAStatUserName         # VPN_NAME
            '9', # SONICWALL-FIREWALL-IP-STATISTICS-MIB::sonicSAStatEncryptByteCount # Byte_Enc
            '11',# SONICWALL-FIREWALL-IP-STATISTICS-MIB::sonicSAStatDecryptByteCount # Byte_Dec
            '2', # SONICWALL-FIREWALL-IP-STATISTICS-MIB::sonicSAStatPeerGateway      # RemoteIP
            ]),
       parse_function=parse_sonicwall_vpn,
)

register.check_plugin(
        name="sonicwall_vpn",
        service_name="SW %s",
        discovery_function=discovery_sonicwall_vpn,
        check_function=check_sonicwall_vpn,
)
